/**********************************************************************
 * Copyright (c) 2011, 2014 IBM Corporation
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.execution.local.util;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.tptp.platform.iac.administrator.internal.common.CommonConstants;

public class ServerReachUtils {

	/**
	 * @return Return a java.util.List of all InetAddresses for each network interface on the local machine.
	 */
	public static List<InetAddress> getLocalHostAddresses()  {
		ArrayList<InetAddress> result = new ArrayList<InetAddress>();
		
		NetworkInterface ni = null;
		try {
			for (Enumeration<NetworkInterface> ifaces = NetworkInterface.getNetworkInterfaces(); ifaces.hasMoreElements();) {
				ni = (NetworkInterface) ifaces.nextElement();
				InetAddress ia = null;
				
				for (Enumeration<InetAddress> ipaddr = ni.getInetAddresses(); ipaddr.hasMoreElements();) {
					
					ia = (InetAddress) ipaddr.nextElement();
					
					result.add(ia);
				}
			}
		} catch(SocketException se) { /* Ignore this error */}
		
		return result;
	}
	
	/**
	 * Make a guess at the correct externally accessible InetAddress for this host. This method
	 * is not guaranteed to provide an externally accessible address; it is also not guaranteed 
	 * to return anything (i.e. it may return false).
	 * 
	 * @return Best guess at externally accessible InetAddress, or null one could not be determined.
	 */
	public static InetAddress getLocalHostAddress() {
		
		List<InetAddress> ial = getLocalHostAddresses();
		try {
			
			// This call returns only the first portion of your hostname, everything before the first dot
			String shortenedHostName = InetAddress.getLocalHost().getHostName().toLowerCase();
			
			// For each address in the list..
			for(Iterator<InetAddress> i = ial.iterator(); i.hasNext();) {
				InetAddress ia = (InetAddress)i.next(); 

				String iaHN = ia.getHostName().toLowerCase();
				
				// .. If one of the addresses begins with the shortened host name, followed by the separator, we have a match
				if(iaHN.startsWith(shortenedHostName + ".")) {
					return ia;
				}
					
			}
			
		} catch (Exception e) {
			/* Any exceptions here will return null, reverting the calling function to its previous behaviour */
		}
		
		return null;
	}

	private static IEclipsePreferences openACPrefs() {

		IEclipsePreferences preferences = InstanceScope.INSTANCE.getNode("org.eclipse.tptp.platform.iac.administrator");
		return preferences;
	}
	
	/** Whether or not the user has selected the 'using a firewall in my workbench' setting in the Eclipse UI. */
	public static boolean isFirewallSettingEnabledInUI() {

		IEclipsePreferences store = openACPrefs();
		if(store != null) {
			return store.getBoolean(CommonConstants.DATAPOOL_PORT_ENABLED, false);
		}
		return false;
	}
	
	/** If the user has checked the 'workbench behind firewall' checkbox, and the 'use random port' field
	 * is not selected, then attempt to parse the text field and return that value.  */
	public static int retrServSockStartFromUI() {
		
		int result = 0;
		
		IEclipsePreferences store = openACPrefs();
		if(store == null) {
			return result;
		}
		
		boolean firewallCheckBoxEnabled = store.getBoolean(CommonConstants.DATAPOOL_PORT_ENABLED, false);
		boolean randomPortSelected = store.getBoolean(CommonConstants.DATAPOOL_PORT_RANDOM, true);
		String customPortValue = store.get(CommonConstants.DATAPOOL_PORT_NUMBER, CommonConstants.EMPTY_STRING);
		
		if(firewallCheckBoxEnabled && !randomPortSelected) {
			try {
				result = Integer.parseInt(customPortValue);
			} catch(NumberFormatException e) { /* ignore exception, result is 0 */}
		}
		
		return result;
	}

	private static boolean isLocalAddressEnabledInUI( IEclipsePreferences store ) {
		
		if( null == store )
			return false;
		
		return store.getBoolean(CommonConstants.DATAPOOL_REACHBACK_ADDR_ENABLED, false);
		
	}
	
	private static String retrLocalAddressFromUI( IEclipsePreferences store ) {

		if( null == store )
			return null;
		return store.get(CommonConstants.DATAPOOL_REACHBACK_ADDR, CommonConstants.EMPTY_STRING);
		
	}

	public static InetAddress getReachableLocalAddress() {
		
		InetAddress addr = null;

		IEclipsePreferences prefs = openACPrefs();
		if( isLocalAddressEnabledInUI(prefs) ) {
			try {
				addr = InetAddress.getByName(retrLocalAddressFromUI(prefs));
			} catch( UnknownHostException e ) {
				// Ok, we'll fall through and try to auto-detect the addr
				addr = null;
			}
		}
		
		// Address not specified by user, try and auto-detect 
		if( null == addr ){
			try {
				InetAddress localAddress = ServerReachUtils.getLocalHostAddress();
				if( null == localAddress )
					addr = InetAddress.getLocalHost();
				addr = localAddress;
			} catch( UnknownHostException e ) {
				// Can't do much about this
				addr = null;
			}
		}
		
		return addr;
	}
}
