/*****************************************************************************
 * Copyright (c) 2012 IBM.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 *
 * $Id$
 *****************************************************************************/

#include <assert.h>
#include <errno.h>
#include "OSAL.h"

Martini::OSA::CRWLock::CRWLock() : m_depth(0)
{
	memset( &m_owner, 0, sizeof(m_owner) );
}

Martini::OSA::CRWLock::~CRWLock()
{
	pthread_rwlock_destroy(&m_pthreadRwlock);
}


TResult Martini::OSA::CRWLock::EnterReader()
{

	// If we own the write lock, just increment the depth and return.
	if( pthread_equal( pthread_self(), m_owner ) > 0 ) {

		atomicInc( &m_depth );
		return MRTE_RESULT_OK;

	}

	// Otherwise do a normal rd-acquire
	if( pthread_rwlock_rdlock(&m_pthreadRwlock) < 0 ) {
		
		assert( EDEADLK != errno );
		return MRTE_ERROR_LIBRARY_FAILURE;
		
	}

	atomicInc( &m_depth );
	return MRTE_RESULT_OK;

}

TResult Martini::OSA::CRWLock::LeaveReader()
{

	// If we own the write-lock, just decrement the depth and return
	if( pthread_equal( pthread_self(), m_owner ) > 0 ) {

		int newDepth = atomicDec( &m_depth );

		// When nesting locks, it is invalid for the outer lock to be a read-lock;
		// therefore the depth after decrementing must be greater than 0.
		assert( newDepth > 0 );
		return MRTE_RESULT_OK;

	}

	// Otherwise do a normal release.
	if( pthread_rwlock_unlock(&m_pthreadRwlock) < 0 ) {

		assert( EPERM != errno );
		assert( EINVAL != errno );
		return MRTE_ERROR_LIBRARY_FAILURE;

	}

	atomicDec( &m_depth );
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::EnterWriter()
{

	// Recursive acquire, just increment the count and return
	if( pthread_equal( pthread_self(), m_owner ) > 0 ) {
		atomicInc( &m_depth );
		return MRTE_RESULT_OK;
	}

	// Otherwise do a normal wr-acquire
	if( pthread_rwlock_wrlock(&m_pthreadRwlock) < 0 ) {
		
		assert( EDEADLK != errno );
		return MRTE_ERROR_LIBRARY_FAILURE;
		
	}
	assert( 0 == m_depth );

	// Record our ownership and bump the count
	m_owner = pthread_self();
	atomicInc( &m_depth );
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::LeaveWriter()
{
	assert( pthread_equal( pthread_self(), m_owner ) > 0 );

	// Decrement the depth. If we are the outer 
	// frame then we release the lock entirely
	if( 0 == atomicDec( &m_depth ) ) {

		memset( &m_owner, 0, sizeof(m_owner) );
		if( pthread_rwlock_unlock(&m_pthreadRwlock) < 0 ) {
			
			assert( EPERM != errno );
			assert( EINVAL != errno );
			
		}

	}

	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::Destroy()
{
	delete this;
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::Create()
{
	pthread_rwlockattr_t attr;
	
	pthread_rwlockattr_init(&attr);
	// PTHREAD_RWLOCK_PREFER_READER_NP
	// PTHREAD_RWLOCK_PREFER_WRITEr_NP
	// PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP
	// PTHREAD_RWLOCK_DEFAULT_NP = PTHREAD_RWLOCK_PREFER_READER_NP
//	pthread_rwlockattr_setkind_np( &attr, PTHREAD_RWLOCK_PREFER_WRITER_NP );
	pthread_rwlock_init(&m_pthreadRwlock, &attr);	// always returns 0
	
	return MRTE_RESULT_OK;
}

